/***************************************************************************
                          kdrawchemdoc.h  -  description
                             -------------------
    begin                : Mon Jul 15 12:24:12 CDT 2002
    copyright            : (C) 2002 by Adam Tenderholt
    email                : a-tenderholt@northwestern.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KDRAWCHEMDOC_H
#define KDRAWCHEMDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// include files for KDE
#include <kurl.h>
#include "chemdata.h"

// forward declaration of the KDrawChem classes
class KDrawChemView;

/**	KDrawChemDoc provides a document object for a document-view model.
  *
  * The KDrawChemDoc class provides a document object that can be used in conjunction with the classes KDrawChemApp and KDrawChemView
  * to create a document-view model for standard KDE applications based on KApplication and KMainWindow. Thereby, the document object
  * is created by the KDrawChemApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KDrawChemView objects. Also, KDrawChemDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.2 code generation
  */
class KDrawChemDoc : public ChemData
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    KDrawChemDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KDrawChemDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KDrawChemView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KDrawChemView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const KURL& url, const char *format=0);
    /** imports file into current document using OpenBabel */
    bool import(const KURL& url, const char* format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const KURL& url, const char *format=0);
    /** returns the KURL of the document */
    const KURL& URL() const;
    /** sets the URL of the document */
	  void setURL(const KURL& url);
	
  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KDrawChemView *sender);
  /** Sets the modified flag to true */
  void slotModified();
  /** No descriptions */
  void slotFilterChanged(const QString& filter);
 	
  public:	
    /** the list of the views currently connected to the document */
    static QList<KDrawChemView> *pViewList;	

  private:
    /** the modified flag of the current document */
  /**  */
  QString myFilter;
    bool modified;
    KURL doc_url;

};

#endif // KDRAWCHEMDOC_H
