/* -*- C++ -*-

Copyright (C) 2000-2001 Francesco Bresciani <fbresciani@users.sourceforge.net>
This file is part of Kemistry

Kemistry is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Kemistry is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Kemistry; see the file COPYING.  If not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef KMOLCALCPARSER_H
#define KMOLCALCPARSER_H

#ifdef HAVE_CONFIG_H

#include <config.h>
#endif

#include <qdict.h>
#include <qmap.h>

#include "kmolcalcdata.h"

    class KMolCalcParser
    {
    public:
	KMolCalcParser();
	virtual ~KMolCalcParser();

	void parseFormula(const QString& formula, bool onlyElements = false);
	QString parseFormulaToString(const QString& formula,
                                     bool onlyElements = true);

	void parseFile(const QString& filename);
	double computeMass();
	void computeMassAndElemPercent(double &mass, 
			QMap<QString, double>& percent);

	QMap<QString, double> elementsCount()
            {return substituteGroup(_subUnitsCount);}

	QString errorMessage()
	{
		QString msg = _error;
		_error = QString::null;
		return msg;
	}


    private:
        QMap<QString, double> _subUnitsCount;
        QString _error;

    private:
        QMap<QString, double> substituteGroup(
				const QMap<QString, double>& groupMap);
        bool parseString(const QString& formula, QMap<QString, double>& count);
    };

#endif // KMOLCALCPARSER_H
